#// File    : CUSTMVAL.MAK
#// Author  : Eric Woodruff,  CIS ID: 72134,1150
#// Updated : Tue 01/24/95 20:37:28
#// Note    : Hereby declared public domain
#// Compiler: Borland C++ 3.1 to 4.xx
#//
#   Make file for building the custom validator library.  Unless overridden by
#   command line flags, this will generate a real mode, non-overlay library
#   called CUSTMVAL.LIB.
#
# File location assumptions:
#
#       All files are in the current directory and the makefile is
#       ran from the same directory.
#
# MAKE with:
#
# No command line flags:
#       Compile for Borland C++ 4.xx using Turbo Vision 2.0
#       Directories: Compiler Root: C:\BC4
#                          Include: C:\BC4\INCLUDE
#                       TV Include: C:\BC4\INCLUDE\TVISION
#
# -DBC31
#       Compile for Borland C++ 3.1 using Turbo Vision 2.0
#       Directories: Compiler Root: C:\BC
#                          Include: C:\BC\INCLUDE
#                       TV Include: C:\BC\INCLUDE\TVISION
#
# Flags for MAKEFILE.MAK:
#
# These command line switches alter the build flags used during
# compilation.  You can use them to alter the build characteristics or you
# can edit the default option flag settings in the make file down below.
#
#     -DBC31         Build using Borland C++ 3.1
#                    If not specified, Borland C++ 4.xx is assumed
#
#     -DCPU286       Build library for the 80286 and up (CPUFLAG = -2)
#                    If not specified, compile for 8088 (CPUFLAG = -1- -2-)
#                    NOTE:
#                       If -DDOS16 (DPMI16), automatic CPUFLAG = -2
#                       If -DDOS32 (DPMI32), automatic CPUFLAG = -3
#
#     -DOPT32        Compiler bugs prevent use of the optimization options
#                    for 32-bit protected mode apps prior to BC++ 4.5.
#                    BC++ 4.5 fixes the bugs, so define OPT32 to utilize
#                    the optimization features of the 32-bit compiler.
#
#     -DOVERLAY      Build an overlayable version  - CUSTMVLO.LIB
#
#     -DDOS16        Build the 16-bit DPMI version - CUSTVL16.LIB
#                    BC++ 4.xx and Power Pack required
#
#     -DDOS32        Build the 32-bit DPMI version - CUSTVL32.LIB
#                    BC++ 4.xx and Power Pack required
#
#     -DOPT32        Compiler bugs prevent use of the optimization options
#                    for 32-bit protected mode apps prior to BC++ 4.5.
#                    BC++ 4.5 fixes the bugs, so define OPT32 to utilize
#                    the optimization features of the 32-bit compiler.
#
#     -DDEBUG        Include full debug info in the library.
#                    Library name generated depends on the above settings:
#                       CUSTMVLD.LIB             - Real mode, non overlay
#                       CUSTVLOD.LIB             - Real mode, overlay
#                       CUSTV16D.LIB             - 16-bit protected mode
#                       CUSTV32D.LIB             - 32-bit protected mode
#
#     -DBCROOT=...   Specifies the path where you installed the compiler
#                    (i.e. C:\BC, C:\BC4, C:\BORLANDC, etc.)
#
#     -DTVINCDIR=... Specifies the path where you installed Turbo Vision's
#                    header files
#                    (i.e. C:\BC\TVISION\INCLUDE, C:\AF\INCLUDE, etc.)
#
#     -DALIGNMENT=2  Optional word (=2) or dword (=4) aligned library.  Only
#     -DALIGNMENT=4  use it if you have a word or dword aligned TV library!
#
# Examples:
#     make                    - Real mode (CUSTMVAL.LIB) for BC++ 4.xx
#     make -DOVERLAY          - Overlay (CUSTMVLO.LIB) for BC++ 4.xx
#     make -DBC31             - Real mode (CUSTMVAL.LIB) for BC++ 3.1
#     make -DBC31 -DOVERLAY   - Overlay (CUSTMVLO.LIB) for BC++ 3.1
#
#     make -DCPU286           - Real mode, BC++ 4.xx, compiled for 286
#     make -DOVERLAY -DCPU286 - Overlay library, BC++ 4.xx, for 286
#     make -DDOS16            - 16 bit DPMI
#     make -DDOS32            - 32 bit DPMI
#     make -DDOS32 -DDEBUG    - 32 bit DPMI with full debug info
#

.autodepend
.swap

# ****************** Uncomment the appropriate lines to default to a
# ****************** specific build mode.

# The value assigned is not important.  It's just to get the defintion
# into existence.  If all are commented out, it defaults to BC++ 4.xx
# with Turbo Vision 2.0.

# BC31 =
# OPT32 =

# ****************** Here's where to adjust the base directory location

!if !$d(BCROOT)
!if $d(BC31)
BCROOT = C:\BC
!else
BCROOT = C:\BC4
!endif
!endif

# ****************** Here's where to adjust the TV include file location

!if !$d(TVINCDIR)
TVINCDIR = $(BCROOT)\INCLUDE\TVISION
!endif

# Set include file locations and default response file name.
INCLUDE = $(BCROOT)\INCLUDE;$(TVINCDIR);.

RSPNAME = CUSTMVAL.RSP

# ******************* Set up for 32 bit protected mode compile
!if $d(DOS32)

!if $d(DEBUG)
CFGNAME = CUSTV32D.CFG
LIBNAME = CUSTV32D.LIB
OBJDIR  = OBJS32D
!else
CFGNAME = CUSTVL32.CFG
LIBNAME = CUSTVL32.LIB
OBJDIR  = OBJS32
!endif

CPUFLAG = -3
TARGETFLAG = -WX

CC      = BCC32
LIB     = TLIB

!else   # DOS32

# ******************* Set up for 16 bit protected mode compile
!if $d(DOS16)

!if $d(DEBUG)
CFGNAME = CUSTV16D.CFG
LIBNAME = CUSTV16D.LIB
OBJDIR  = OBJS16D
!else
CFGNAME = CUSTVL16.CFG
LIBNAME = CUSTVL16.LIB
OBJDIR  = OBJS16
!endif

CPUFLAG = -2
TARGETFLAG = -WX -ml

CC      = BCC
LIB     = TLIB

!else   # DOS16
# ******************* Set up for real mode compile

!if $d(DEBUG)

!if $d(OVERLAY)
CFGNAME = CUSTVLOD.CFG
LIBNAME = CUSTVLOD.LIB
OBJDIR  = OBJSOD
!else
CFGNAME = CUSTMVLD.CFG
LIBNAME = CUSTMVLD.LIB
OBJDIR  = OBJSD
!endif

!else

!if $d(OVERLAY)
CFGNAME = CUSTMVLO.CFG
LIBNAME = CUSTMVLO.LIB
OBJDIR  = OBJSO
!else
CFGNAME = CUSTMVAL.CFG
LIBNAME = CUSTMVAL.LIB
OBJDIR  = OBJS
!endif

!endif

# Set CPU type being compiled for in real mode.
!if $d(CPU286)
CPUFLAG = -2
!else
CPUFLAG = -1- -2-
!endif

TARGETFLAG = -ml

CC      = BCC
LIB     = TLIB

!endif # DOS16
!endif # DOS32

# ******************* Set other compiler options

!if $d(OVERLAY)
!if !$d(DOS16) && !$d(DOS32)
!if $d(BC31)
CCOVYFLAGS = -Y -Vs -Vb -B
AOVYFLAGS  = /o
!else
CCOVYFLAGS = -Y -Vs
AOVYFLAGS  = /o
!endif
!else
!error There's no need to use overlays with the 16-bit or 32-bit DPMI versions.
!endif
!endif

!if $d(ALIGNMENT)
ALIGNFLAG = -a$(ALIGNMENT)
!else
ALIGNFLAG =
!endif

# ******************* Set debug and optimization options

CFLAGS1    =
CFLAGSN32  =

!if $d(DEBUG)

LIBFLAGS   = /P128
CDEBUGFLAG = /v /DDIAGNOSTICS -k -N

!else

LIBFLAGS   = /0
CDEBUGFLAG = /v- /DNDEBUG -k- -N- -G-

# These optimization options **cannot** be used for the BC++ 4.0/4.02
# 32-bit protected mode compiler due to bugs.   However, BC++ 4.5 fixes
# these bugs, so define OPT32 to include these optimizations if you have it.
!if !$d(DOS32) || $d(OPT32)
CFLAGSN32 = -Oe -Os -O -Ob -Ol -Z
!endif

!if !$d(DOS32)
CFLAGSN32 = -h
!endif

!endif

# Turn off exceptions and RTTI if using BC++ 4.xx and above
!if !$d(BC31)
CFLAGS1 = $(CFLAGS1) -RT- -x-
!endif

# Put them all together.  Compiler options are spread across three definitions
# because they can exceed 128 characters per line which causes it to fail.
CFLAGS1 = $(CFLAGS1) -ff -w -w-sig -w-amp -P -n$(OBJDIR)
CFLAGS2 = $(CDEBUGFLAG) $(ALIGNFLAG) $(CCOVYFLAGS) $(CPUFLAG) $(CFLAGSN32)
CFLAGS3 = -I$(INCLUDE)

# ******************* Build dependencies

.path.obj = $(OBJDIR)

OBJS = CSTMSGVL.OBJ \
       MTPICVAL.OBJ \
       NMCUSTVL.OBJ \
       SCUSTVAL.OBJ \
       TDBLVAL.OBJ \
       TNUMVAL.OBJ \
       ICSTRVAL.OBJ \
       NMFMTNUM.OBJ \
       SFMTNUM.OBJ \
       TFMTNUM.OBJ \
       ULONGRV.OBJ

# Go ahead and build it.

.cpp.obj:
     $(CC) -c +$(CFGNAME) {$. }

all: $(CFGNAME) dirs $(LIBNAME)
    DEL $(CFGNAME)

dirs:
    IF NOT EXIST $(OBJDIR) MD $(OBJDIR)

$(CFGNAME):
    ECHO $(TARGETFLAG) $(CFLAGS1) > $(CFGNAME)
    ECHO $(CFLAGS2) >> $(CFGNAME)
    ECHO $(CFLAGS3) >> $(CFGNAME)

$(LIBNAME): $(OBJS)
     @cd $(OBJDIR)
     @if exist ..\$(LIBNAME) del ..\$(LIBNAME)
     $(LIB) $(LIBFLAGS) ..\$(LIBNAME) @..\$(RSPNAME)
     @cd ..

# End of Custom Validator Library make file
